#ifndef COMMANDLINEPARSER_H
#define COMMANDLINEPARSER_H

#include <string>
#include <map>
#include <list>

using namespace std;

/**
 * @brief This class provides an easy-to-use command line argument parser.
 *
 *
 *
 * Example program:
 \code
#include <iostream>
#include <new>
#include <string>
#include "CommandLineParser.h"

using namespace std;

int main(int argc, char **argv)
{
	try{
		CommandLineParser parser("CommandLineParser class example program.");
		parser.addParameter("-name","optional string parameter","MyName");
		parser.addParameter("I","required integer parameter");
		parser.addParameter("-OptionalInteger","optional integer parameter","1");
		parser.addParameter("D","required double parameter");
		parser.addParameter("S","command line switch");
		parser.addParameter("iV","integer parameter (e.g. 0,1,2,3)");
		parser.parse(argc,argv);
		cout << "--name argument: " << parser.getArgument("-name") << endl
			 << "-I (integer) : " << parser.getLongIntArgument("I") << endl
			 << "--OptionalInteger : "
			 << parser.getLongIntArgument("-OptionalInteger") << endl
			 << "-D (double) : " << parser.getDoubleArgument("D") << endl
			 << "switch S "
			 << ((parser.isSet("S")) ? "has" : "hasn't")
			 << " been set." << endl;
		list<int> intlist = parser.getIntegerListArgument("iV");
		for(list<int>::const_iterator it=intlist.begin(); it != intlist.end(); it++)
		{
			cout << " integer list entry:  " << *it << endl;
		}
		cout << "remaining arguments: " << endl;
	    const list<string> &remaining_args = parser.getRemainingArguments();
	    for(list<string>::const_iterator it=remaining_args.begin();
	      it != remaining_args.end(); it++)
	    {
	      cout << *it << endl;
	    }
	}
	catch(string msg)
	{
		cerr << msg << endl;
		return -1;
	}
	catch(bad_alloc e)
	{
		cerr << "Memory allocation failed!" << endl
			 << e.what() << endl;
		return -2;
	}
	catch(...)
	{
		cerr << "An unknown exception occured!" << endl;
		return -3;
	}
	return 0;
}
 \endcode
 *
 * When calling the program with the -h switch it prints out:
 \verbatim
CommandLineParser class example program.
Command line switches:
  -h                              - help
  --name                          - optional string parameter (default value: MyName )
  -I                              - required integer parameter
  --OptionalInteger               - optional integer parameter (default value: 1.0 )
  -D                              - required double parameter
  -S                              - command line switch
 \endverbatim
 * Full example:
 \verbatim
 ./CommandLineParser -I 2 -D 2.1 -S -iV 0,2,4,6 file1 file2
  --name argument: MyName
  -I (integer) : 2
  --OptionalInteger : 1
  -D (double) : 2.1
  switch S has been set.
   integer list entry:  0
   integer list entry:  2
   integer list entry:  4
   integer list entry:  6
 \endverbatim
 * @author Ruediger Knoerig
 */
class CommandLineParser{
public:
	/**
	 * default constructor
	 * @param description_text text describing the function of the program
	 * @param width_tokenfield width of the token field in the help text
	 */
	CommandLineParser(string description_text,int width_tokenfield=30) \
	 : descriptiontext(description_text),
	 tokenfield_width(width_tokenfield)
	{
		descriptiontext += string("\nCommand line switches:\n  -h");
		descriptiontext += string(tokenfield_width-1,' ');
		descriptiontext += string(" - help\n");
	}

	/** destructor */
	virtual ~CommandLineParser() {}

	/**
	 * @brief method for adding a parameter.
	 * @param token the parameter has the command line switch -token
	 * @param helptext helptext describing the function of the switch
	 * @param defaultvalue default value for the parameter.
	 *        Omit for required parameters; for optional parameters enter
	 * 		  at least some nonsene
	 */
	void addParameter(string token,string helptext,string defaultvalue=string());

	/**
	 * @brief method for parsing the command line arguments.
	 * Throws a help string if the -h switch has been activated or zeroArgsForHelp
	 * is true.
	 * @param argc number of strings in the array argv
	 * @param argv Array of C-strings with the command line parameters.
	 * @param zeroArgsForHelp set to true if the parser should interpret a
	 * call of the program without args as a call with the -h option
	 */
	void parse(int argc, char **argv,bool zeroArgsForHelp=false) throw(string);

	/**
	 * @brief method for retrieving a string argument.
	 * Throws a string exception if a non-optional parameter has not being set.
	 * @param parameter token of the parameter to retrieve
	 * @return assigned argument for the parameter
	 */
	string getArgument(string parameter) throw(string);

	/**
	 * method returning true if an parameter has been set
	 * @param parameter token of the parameter to retrieve
	 * @return true if the command line switch was in use
	 */
	bool isSet(string parameter) { return !tokenvaluemap[parameter].empty(); }

	/**
	 * @brief method for retrieving an integer argument.
	 * Throws a string exception if a non-optional parameter has not been set
	 * or when the conversion to a number value fails.
	 * @param parameter token of the parameter to retrieve
	 * @param base base of the number system (must be between 2 and 36, for default: 10)
	 * @return assigned argument for the parameter
	 */
	long int getLongIntArgument(string parameter,int base=0) throw(string);

        /**
	 * @brief method for retrieving a floating-point argument.
	 * Throws a string exception if a non-optional parameter has not been set
	 * or when the conversion to a number value fails.
	 * @param parameter token of the parameter to retrieve
	 * @return assigned argument for the parameter
	 */
	double getDoubleArgument(string parameter) throw(string);

        /**
	 * @brief method for retrieving a boolean argument.
	 * Throws a string exception if a non-optional parameter has not been set
	 * @param parameter token of the parameter to retrieve
	 * @return assigned argument for the parameter
	 */
	bool getBooleanArgument(string parameter) throw(string);

	/**
	 *  @brief method for retrieving a list of integer values.
	 * Throws a string exception if a non-optional parameter has not been set
	 * or when the conversion to a number value fails.
	 * @param parameter token of the parameter to retrieve.
	 * @return assigned argument for the parameter.
	 */
	list<int> getIntegerListArgument(string parameter) throw(string);

	/**
	 *  @brief method for retrieving a list of double values.
	 * Throws a string exception if a non-optional parameter has not been set
	 * or when the conversion to a number value fails.
	 * @param parameter token of the parameter to retrieve.
	 * @return assigned argument for the parameter.
	 */
	list<double> getDoubleListArgument(string parameter) throw(string);

	/**
	 * @brief Method for getting the help text
	 * @return help message
	 */
	string getHelpMessage() const { return descriptiontext; }

	/**
	 * @brief Method for getting the list of remaining arguments after the last
	 * parameter argument.
	 */
	const list<string> &getRemainingArguments() const { return remaining_tokens; }

private: // private attributes
	/** association between parameter token and values */
	map<string,string> tokenvaluemap;
	/** string describing the function of the program */
	string descriptiontext;
	/** width of the token field */
	const int tokenfield_width;
	/** remaining unclassified tokens */
	list<string> remaining_tokens;
};

#endif // COMMANDLINEPARSER_H
